﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which manipulates two integer values. TODO
  /// </summary>
  [Item("UniformIntValueManipulator", "An operator which manipulates two integer values. TODO")]
  [StorableClass]
  public class UniformIntValueManipulator : Item, IIntValueManipulator {
    #region Constructors and Cloning
    public UniformIntValueManipulator() : base() { }

    [StorableConstructor]
    protected UniformIntValueManipulator(bool deserializing) : base(deserializing) { }
    protected UniformIntValueManipulator(UniformIntValueManipulator original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformIntValueManipulator(this, cloner);
    }
    #endregion

    public void Apply(IRandom random, IntValue value, IntValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, IntValue value, IntValueRange range) {
      var vector = new IntegerVector(1);
      var bounds = new IntMatrix(1, 2);
      bounds[0, 0] = range.LowerBound.Value;
      bounds[0, 1] = range.UpperBound.Value;
      int val = value.Value;

      do {
        vector[0] = val;
        UniformOnePositionManipulator.Apply(random, vector, bounds);
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
      } while (!range.IsInRange(value.Value));
    }
  }
}
